@@ -22,6 +22,7 @@ import netmiko
 from neutron_lib import constants as const
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_utils import strutils
 from paramiko import PKey as _pkey  # noqa - This is for a monkeypatch
 import paramiko  # noqa - Must load after the patch
 import tenacity
@@ -78,6 +79,14 @@ class NetmikoSwitch(devices.GenericSwitchDevice):
 
     DELETE_NETWORK = None
 
+    PLUG_SWITCH_TO_NETWORK = None
+
+    UNPLUG_SWITCH_FROM_NETWORK = None
+
+    SHOW_VLAN_PORTS = None
+
+    SHOW_VLAN_VNI = None
+
     PLUG_PORT_TO_NETWORK = None
 
     DELETE_PORT = None
@@ -161,9 +170,17 @@ class NetmikoSwitch(devices.GenericSwitchDevice):
             self.config['session_log_record_writes'] = True
             self.config['session_log_file_mode'] = 'append'
 
-        _NUMERIC_CAST = {
+        _NETMIKO_CAST = {
             "port": int,
+            "verbose": bool,
             "global_delay_factor": float,
+            "global_cmd_verify": bool,
+            "use_keys": bool,
+            "disable_sha2_fix": bool,
+            "allow_agent": bool,
+            "ssh_strict": bool,
+            "system_host_keys": bool,
+            "alt_host_keys": bool,
             "conn_timeout": float,
             "auth_timeout": float,
             "banner_timeout": float,
@@ -171,14 +188,24 @@ class NetmikoSwitch(devices.GenericSwitchDevice):
             "timeout": float,
             "session_timeout": float,
             "read_timeout_override": float,
-            "keepalive": int,
+            "keepalive": float,
+            "fast_cli": bool,
+            "_legacy_mode": bool,
+            "session_log_record_writes": bool,
+            "allow_auto_change": bool,
+            "auto_connect": bool,
+            "delay_factor_compat": bool,
+            "disable_lf_normalization": bool,
         }
 
-        for key, expected_type in _NUMERIC_CAST.items():
+        for key, expected_type in _NETMIKO_CAST.items():
             value = self.config.get(key)
             if isinstance(value, str):
                 try:
-                    self.config[key] = expected_type(value)
+                    if expected_type == bool:
+                        self.config[key] = strutils.bool_from_string(value)
+                    else:
+                        self.config[key] = expected_type(value)
                 except ValueError:
                     LOG.error(
                         "Invalid value %s for %s; expected %s",
@@ -214,6 +241,11 @@ class NetmikoSwitch(devices.GenericSwitchDevice):
                 ('ngs-' + device_utils.get_hostname()).encode('ascii'))
             self.locker.start()
             atexit.register(self.locker.stop)
+        else:
+            LOG.warning(
+                "Switch %s: [ngs_coordination] backend_url is not "
+                "configured. The ngs_max_connections is ignored.",
+                self.lock_kwargs['locks_prefix'])
 
     @property
     def support_trunk_on_ports(self):
@@ -364,17 +396,135 @@ class NetmikoSwitch(devices.GenericSwitchDevice):
                                       network_name=network_name)
         return self.send_commands_to_device(cmds)
 
+    @check_output('plug vni')
+    def plug_switch_to_network(self, vni: int, segmentation_id: int,
+                               physnet: str = None):
+        # NOTE: PLUG_SWITCH_TO_NETWORK commands typically start with
+        # 'vlan {segmentation_id}' which creates the VLAN if it doesn't
+        # exist. This is necessary for L2VNI scenarios where the VLAN
+        # segment may be dynamically allocated and not created via
+        # add_network().
+        #
+        # physnet parameter added for per-physnet mcast-group resolution
+        # in vendor-specific implementations (e.g., Cisco NX-OS).
+        # Devices that don't need physnet can ignore this parameter.
+        cmds = [
+            self._format_commands(
+                self.PLUG_SWITCH_TO_NETWORK,
+                vni=vni,
+                segmentation_id=segmentation_id)
+        ]
+
+        return self.send_commands_to_device(cmds)
+
+    @check_output('unplug vni')
+    def unplug_switch_from_network(self, vni: int, segmentation_id: int,
+                                   physnet: str = None):
+        # physnet parameter added for signature consistency with
+        # plug_switch_to_network. Most implementations won't need it.
+        cmds = [
+            self._format_commands(
+                self.UNPLUG_SWITCH_FROM_NETWORK,
+                vni=vni,
+                segmentation_id=segmentation_id)
+        ]
+
+        return self.send_commands_to_device(cmds)
+
+    def vlan_has_ports(self, segmentation_id: int) -> bool:
+        """Check if a VLAN has any ports attached.
+
+        :param segmentation_id: VLAN identifier to check
+        :returns: True if VLAN has ports, False otherwise.
+        """
+        if not self.SHOW_VLAN_PORTS:
+            # If no command defined, conservatively assume ports exist
+            LOG.warning("SHOW_VLAN_PORTS not implemented for %s, "
+                        "assuming VLAN %s has ports",
+                        self.device_name, segmentation_id)
+            return True
+
+        cmd = self._format_commands(
+            self.SHOW_VLAN_PORTS,
+            segmentation_id=segmentation_id)
+
+        try:
+            with self._get_connection() as net_connect:
+                output = net_connect.send_command(cmd[0])
+                return self._parse_vlan_ports(output, segmentation_id)
+        except Exception as e:
+            LOG.error("Failed to check VLAN %s ports on %s: %s",
+                      segmentation_id, self.device_name, e)
+            # On error, conservatively assume ports exist
+            return True
+
+    def _parse_vlan_ports(self, output: str, segmentation_id: int) -> bool:
+        """Parse output of SHOW_VLAN_PORTS command.
+
+        Subclasses should override this for vendor-specific parsing.
+
+        :param output: Command output from switch
+        :param segmentation_id: VLAN identifier being checked
+        :returns: True if VLAN has ports, False otherwise
+        """
+        # Default implementation: conservatively assume ports exist
+        return True
+
+    def vlan_has_vni(self, segmentation_id: int, vni: int) -> bool:
+        """Check if a VLAN has a specific VNI configured.
+
+        :param segmentation_id: VLAN identifier to check
+        :param vni: VNI to check for
+        :returns: True if VLAN has this VNI configured, False otherwise
+        """
+        if not self.SHOW_VLAN_VNI:
+            # If no command defined, conservatively return False
+            # (assume VNI not configured, will attempt to configure)
+            LOG.warning("SHOW_VLAN_VNI not implemented for %s, "
+                        "cannot check if VNI %s is on VLAN %s",
+                        self.device_name, vni, segmentation_id)
+            return False
+
+        cmd = self._format_commands(
+            self.SHOW_VLAN_VNI,
+            segmentation_id=segmentation_id)
+
+        try:
+            with self._get_connection() as net_connect:
+                output = net_connect.send_command(cmd[0])
+                return self._parse_vlan_vni(output, segmentation_id, vni)
+        except Exception as e:
+            LOG.error("Failed to check VLAN %s VNI %s on %s: %s",
+                      segmentation_id, vni, self.device_name, e)
+            # On error, conservatively return False (will attempt to configure)
+            return False
+
+    def _parse_vlan_vni(self, output: str, segmentation_id: int,
+                        vni: int) -> bool:
+        """Parse output to check if VLAN has VNI configured.
+
+        Subclasses should override this for vendor-specific parsing.
+
+        :param output: Command output from switch
+        :param segmentation_id: VLAN identifier being checked
+        :param vni: VNI to check for
+        :returns: True if VLAN has this VNI, False otherwise
+        """
+        # Default implementation: return False (assume not configured)
+        return False
+
     @check_output('plug port')
-    def plug_port_to_network(self, port, segmentation_id, trunk_details=None):
+    def plug_port_to_network(self, port, segmentation_id, trunk_details=None,
+                             default_vlan=None):
         cmds = []
         if self._disable_inactive_ports() and self.ENABLE_PORT:
             cmds += self._format_commands(self.ENABLE_PORT, port=port)
-        ngs_port_default_vlan = self._get_port_default_vlan()
-        if ngs_port_default_vlan:
+        port_default_vlan = default_vlan or self._get_port_default_vlan()
+        if port_default_vlan:
             cmds += self._format_commands(
                 self.DELETE_PORT,
                 port=port,
-                segmentation_id=ngs_port_default_vlan)
+                segmentation_id=port_default_vlan)
 
         if trunk_details:
             cmds += self._format_commands(self.SET_NATIVE_VLAN,
@@ -393,11 +543,12 @@ class NetmikoSwitch(devices.GenericSwitchDevice):
         return self.send_commands_to_device(cmds)
 
     @check_output('unplug port')
-    def delete_port(self, port, segmentation_id, trunk_details=None):
+    def delete_port(self, port, segmentation_id, trunk_details=None,
+                    default_vlan=None):
         cmds = self._format_commands(self.DELETE_PORT,
                                      port=port,
                                      segmentation_id=segmentation_id)
-        ngs_port_default_vlan = self._get_port_default_vlan()
+        port_default_vlan = default_vlan or self._get_port_default_vlan()
         if trunk_details:
             cmds += self._format_commands(self.DELETE_NATIVE_VLAN,
                                           port=port,
@@ -407,27 +558,28 @@ class NetmikoSwitch(devices.GenericSwitchDevice):
                     self.REMOVE_NETWORK_FROM_TRUNK, port=port,
                     segmentation_id=sub_port['segmentation_id'])
 
-        if ngs_port_default_vlan:
+        if port_default_vlan:
             # NOTE(mgoddard): Pass network_id and segmentation_id for drivers
             # not yet using network_name.
-            network_name = self._get_network_name(ngs_port_default_vlan,
-                                                  ngs_port_default_vlan)
+            network_name = self._get_network_name(port_default_vlan,
+                                                  port_default_vlan)
             cmds += self._format_commands(
                 self.ADD_NETWORK,
-                segmentation_id=ngs_port_default_vlan,
-                network_id=ngs_port_default_vlan,
+                segmentation_id=port_default_vlan,
+                network_id=port_default_vlan,
                 network_name=network_name)
             cmds += self._format_commands(
                 self.PLUG_PORT_TO_NETWORK,
                 port=port,
-                segmentation_id=ngs_port_default_vlan)
+                segmentation_id=port_default_vlan)
         if self._disable_inactive_ports() and self.DISABLE_PORT:
             cmds += self._format_commands(self.DISABLE_PORT, port=port)
 
         return self.send_commands_to_device(cmds)
 
     @check_output('plug bond')
-    def plug_bond_to_network(self, bond, segmentation_id, trunk_details=None):
+    def plug_bond_to_network(self, bond, segmentation_id, trunk_details=None,
+                             default_vlan=None):
         # Fallback to regular plug port if no specialist PLUG_BOND_TO_NETWORK
         # commands set
         if not self.PLUG_BOND_TO_NETWORK:
@@ -436,12 +588,12 @@ class NetmikoSwitch(devices.GenericSwitchDevice):
         cmds = []
         if self._disable_inactive_ports() and self.ENABLE_BOND:
             cmds += self._format_commands(self.ENABLE_BOND, bond=bond)
-        ngs_port_default_vlan = self._get_port_default_vlan()
-        if ngs_port_default_vlan:
+        port_default_vlan = default_vlan or self._get_port_default_vlan()
+        if port_default_vlan:
             cmds += self._format_commands(
                 self.UNPLUG_BOND_FROM_NETWORK,
                 bond=bond,
-                segmentation_id=ngs_port_default_vlan)
+                segmentation_id=port_default_vlan)
 
         if trunk_details:
             cmds += self._format_commands(self.SET_NATIVE_VLAN_BOND,
@@ -461,7 +613,7 @@ class NetmikoSwitch(devices.GenericSwitchDevice):
 
     @check_output('unplug bond')
     def unplug_bond_from_network(self, bond, segmentation_id,
-                                 trunk_details=None):
+                                 trunk_details=None, default_vlan=None):
         # Fallback to regular port delete if no specialist
         # UNPLUG_BOND_FROM_NETWORK commands set
         if not self.UNPLUG_BOND_FROM_NETWORK:
@@ -470,30 +622,30 @@ class NetmikoSwitch(devices.GenericSwitchDevice):
         cmds = self._format_commands(self.UNPLUG_BOND_FROM_NETWORK,
                                      bond=bond,
                                      segmentation_id=segmentation_id)
-        ngs_port_default_vlan = self._get_port_default_vlan()
+        port_default_vlan = default_vlan or self._get_port_default_vlan()
         if trunk_details:
             cmds += self._format_commands(self.DELETE_NATIVE_VLAN_BOND,
                                           bond=bond,
                                           segmentation_id=segmentation_id)
             for sub_port in trunk_details.get('sub_ports', []):
                 cmds += self._format_commands(
-                    self.ADD_NETWORK_TO_BOND_TRUNK, bond=bond,
+                    self.DELETE_NETWORK_ON_BOND_TRUNK, bond=bond,
                     segmentation_id=sub_port['segmentation_id'])
 
-        if ngs_port_default_vlan:
+        if port_default_vlan:
             # NOTE(mgoddard): Pass network_id and segmentation_id for drivers
             # not yet using network_name.
-            network_name = self._get_network_name(ngs_port_default_vlan,
-                                                  ngs_port_default_vlan)
+            network_name = self._get_network_name(port_default_vlan,
+                                                  port_default_vlan)
             cmds += self._format_commands(
                 self.ADD_NETWORK,
-                segmentation_id=ngs_port_default_vlan,
-                network_id=ngs_port_default_vlan,
+                segmentation_id=port_default_vlan,
+                network_id=port_default_vlan,
                 network_name=network_name)
             cmds += self._format_commands(
                 self.PLUG_BOND_TO_NETWORK,
                 bond=bond,
-                segmentation_id=ngs_port_default_vlan)
+                segmentation_id=port_default_vlan)
         if self._disable_inactive_ports() and self.DISABLE_BOND:
             cmds += self._format_commands(self.DISABLE_BOND, bond=bond)
 
@@ -576,8 +728,8 @@ class NetmikoSwitch(devices.GenericSwitchDevice):
     def _get_acl_names(self, sg_id):
         """Get the ACL names for a security group.
 
-        Subclasses should override this method to provide mutiple ACL names on
-        devices which require distinct ACLs for ipv4 vs ipv6 or ingress vs
+        Subclasses should override this method to provide multiple ACL names
+        on devices which require distinct ACLs for ipv4 vs ipv6 or ingress vs
         egress.
 
         :param sg_id: Security group ID
