@@ -372,3 +372,246 @@ error: configuration check-out failed
                          ['delete interfaces 3333 unit 0 '
                           'family ethernet-switching '
                           'vlan members 33'])
+
+    @mock.patch('networking_generic_switch.devices.netmiko_devices.'
+                'NetmikoSwitch.send_commands_to_device',
+                return_value="", autospec=True)
+    @mock.patch('networking_generic_switch.devices.netmiko_devices.'
+                'juniper.Juniper._get_vlan_name_by_id',
+                return_value='vlan100', autospec=True)
+    def test_plug_switch_to_network(self, m_get_vlan_name, mock_exec):
+        self.switch.plug_switch_to_network(5000, 100)
+        m_get_vlan_name.assert_called_with(self.switch, 100)
+        mock_exec.assert_called_with(
+            self.switch,
+            ['set vlans vlan100 vxlan vni 5000'])
+
+    @mock.patch('networking_generic_switch.devices.netmiko_devices.'
+                'NetmikoSwitch.send_commands_to_device',
+                return_value="", autospec=True)
+    @mock.patch('networking_generic_switch.devices.netmiko_devices.'
+                'juniper.Juniper._get_vlan_name_by_id',
+                return_value='my-test-vlan', autospec=True)
+    def test_unplug_switch_from_network(self, m_get_vlan_name, mock_exec):
+        self.switch.unplug_switch_from_network(5000, 100)
+        m_get_vlan_name.assert_called_with(self.switch, 100)
+        mock_exec.assert_called_with(
+            self.switch,
+            ['delete vlans my-test-vlan vxlan vni'])
+
+    def test__parse_vlan_name(self):
+        output = '''Routing instance        VLAN name           Tag     \
+Interfaces
+default-switch          default             1
+default-switch          vlan100             100     xe-0/0/1.0*
+default-switch          vlan200             200'''
+        result = self.switch._parse_vlan_name(output, 100)
+        self.assertEqual('vlan100', result)
+
+    def test__parse_vlan_name_not_found(self):
+        output = '''Routing instance        VLAN name           Tag     \
+Interfaces
+default-switch          default             1
+default-switch          vlan200             200'''
+        result = self.switch._parse_vlan_name(output, 100)
+        self.assertIsNone(result)
+
+    def test__parse_vlan_ports_with_ports(self):
+        output = '''Routing instance        VLAN name           Tag     \
+Interfaces
+default-switch          vlan100             100     xe-0/0/1.0*
+default-switch          vlan200             200'''
+        result = self.switch._parse_vlan_ports(output, 100)
+        self.assertTrue(result)
+
+    def test__parse_vlan_ports_without_ports(self):
+        output = '''Routing instance        VLAN name           Tag     \
+Interfaces
+default-switch          vlan100             100
+default-switch          vlan200             200     xe-0/0/2.0*'''
+        result = self.switch._parse_vlan_ports(output, 100)
+        self.assertFalse(result)
+
+    def test__parse_vlan_ports_vlan_not_found(self):
+        output = '''Routing instance        VLAN name           Tag     \
+Interfaces
+default-switch          vlan200             200     xe-0/0/1.0*'''
+        result = self.switch._parse_vlan_ports(output, 100)
+        self.assertFalse(result)
+
+    def test__parse_vlan_vni_match(self):
+        output = '''Routing instance        VLAN name           Tag     \
+Interfaces
+default-switch          vlan100             100
+  VNI: 5000
+default-switch          vlan200             200
+  VNI: 6000'''
+        result = self.switch._parse_vlan_vni(output, 100, 5000)
+        self.assertTrue(result)
+
+    def test__parse_vlan_vni_no_match(self):
+        output = '''Routing instance        VLAN name           Tag     \
+Interfaces
+default-switch          vlan100             100
+  VNI: 5000
+default-switch          vlan200             200
+  VNI: 6000'''
+        result = self.switch._parse_vlan_vni(output, 100, 9999)
+        self.assertFalse(result)
+
+    def test__parse_vlan_vni_no_vni_configured(self):
+        output = '''Routing instance        VLAN name           Tag     \
+Interfaces
+default-switch          vlan100             100     xe-0/0/1.0*
+default-switch          vlan200             200'''
+        result = self.switch._parse_vlan_vni(output, 100, 5000)
+        self.assertFalse(result)
+
+    def test__parse_vlan_vni_vlan_not_found(self):
+        output = '''Routing instance        VLAN name           Tag     \
+Interfaces
+default-switch          vlan200             200
+  VNI: 6000'''
+        result = self.switch._parse_vlan_vni(output, 100, 5000)
+        self.assertFalse(result)
+
+    # EVPN configuration tests
+    def test_init_default_evpn_config(self):
+        """Test __init__ with default (disabled) EVPN VNI config."""
+        device_cfg = {
+            'device_type': 'netmiko_juniper'
+        }
+        switch = juniper.Juniper(device_cfg)
+        self.assertFalse(switch.evpn_vni_config)
+        self.assertIsNone(switch.bgp_asn)
+
+    def test_init_evpn_vni_config_enabled(self):
+        """Test __init__ with EVPN VNI config enabled."""
+        device_cfg = {
+            'device_type': 'netmiko_juniper',
+            'ngs_evpn_vni_config': 'true',
+            'ngs_bgp_asn': '65000'
+        }
+        switch = juniper.Juniper(device_cfg)
+        self.assertTrue(switch.evpn_vni_config)
+        self.assertEqual(switch.bgp_asn, '65000')
+
+    def test_init_evpn_vni_config_various_values(self):
+        """Test __init__ with various EVPN config values."""
+        # Test 'yes'
+        switch1 = juniper.Juniper({
+            'device_type': 'netmiko_juniper',
+            'ngs_evpn_vni_config': 'yes'})
+        self.assertTrue(switch1.evpn_vni_config)
+
+        # Test '1'
+        switch2 = juniper.Juniper({
+            'device_type': 'netmiko_juniper',
+            'ngs_evpn_vni_config': '1'})
+        self.assertTrue(switch2.evpn_vni_config)
+
+        # Test 'false'
+        switch3 = juniper.Juniper({
+            'device_type': 'netmiko_juniper',
+            'ngs_evpn_vni_config': 'false'})
+        self.assertFalse(switch3.evpn_vni_config)
+
+    @mock.patch('networking_generic_switch.devices.netmiko_devices.'
+                'NetmikoSwitch.send_commands_to_device',
+                return_value='', autospec=True)
+    @mock.patch.object(juniper.Juniper, '_get_vlan_name_by_id',
+                       return_value='vlan100', autospec=True)
+    def test_plug_switch_to_network_with_evpn_enabled(
+            self, mock_get_vlan, mock_exec):
+        """Test plug_switch_to_network with EVPN enabled."""
+        device_cfg = {
+            'device_type': 'netmiko_juniper',
+            'ngs_evpn_vni_config': 'true',
+            'ngs_bgp_asn': '65000'
+        }
+        switch = juniper.Juniper(device_cfg)
+        switch.plug_switch_to_network(10100, 100)
+        mock_exec.assert_called_with(
+            switch,
+            ['set vlans vlan100 vxlan vni 10100',
+             'set vlans vlan100 vrf-target target:65000:10100'])
+
+    @mock.patch('networking_generic_switch.devices.netmiko_devices.'
+                'NetmikoSwitch.send_commands_to_device',
+                return_value='', autospec=True)
+    @mock.patch.object(juniper.Juniper, '_get_vlan_name_by_id',
+                       return_value='vlan100', autospec=True)
+    def test_plug_switch_to_network_without_evpn(
+            self, mock_get_vlan, mock_exec):
+        """Test plug_switch_to_network without EVPN."""
+        device_cfg = {
+            'device_type': 'netmiko_juniper'
+        }
+        switch = juniper.Juniper(device_cfg)
+        switch.plug_switch_to_network(10100, 100)
+        mock_exec.assert_called_with(
+            switch,
+            ['set vlans vlan100 vxlan vni 10100'])
+
+    @mock.patch.object(juniper.Juniper, '_get_vlan_name_by_id',
+                       return_value='vlan100', autospec=True)
+    def test_plug_switch_to_network_evpn_without_bgp_asn(
+            self, mock_get_vlan):
+        """Test plug_switch_to_network fails when EVPN enabled, no ASN."""
+        device_cfg = {
+            'device_type': 'netmiko_juniper',
+            'ngs_evpn_vni_config': 'true'
+        }
+        switch = juniper.Juniper(device_cfg)
+        self.assertRaises(exc.GenericSwitchNetmikoConfigError,
+                          switch.plug_switch_to_network, 10100, 100)
+
+    @mock.patch('networking_generic_switch.devices.netmiko_devices.'
+                'NetmikoSwitch.send_commands_to_device',
+                return_value='', autospec=True)
+    @mock.patch.object(juniper.Juniper, '_get_vlan_name_by_id',
+                       return_value='vlan100', autospec=True)
+    def test_unplug_switch_from_network_with_evpn_enabled(
+            self, mock_get_vlan, mock_exec):
+        """Test unplug_switch_from_network with EVPN enabled."""
+        device_cfg = {
+            'device_type': 'netmiko_juniper',
+            'ngs_evpn_vni_config': 'true',
+            'ngs_bgp_asn': '65000'
+        }
+        switch = juniper.Juniper(device_cfg)
+        switch.unplug_switch_from_network(10100, 100)
+        mock_exec.assert_called_with(
+            switch,
+            ['delete vlans vlan100 vxlan vni',
+             'delete vlans vlan100 vrf-target'])
+
+    @mock.patch('networking_generic_switch.devices.netmiko_devices.'
+                'NetmikoSwitch.send_commands_to_device',
+                return_value='', autospec=True)
+    @mock.patch.object(juniper.Juniper, '_get_vlan_name_by_id',
+                       return_value='vlan100', autospec=True)
+    def test_unplug_switch_from_network_without_evpn(
+            self, mock_get_vlan, mock_exec):
+        """Test unplug_switch_from_network without EVPN."""
+        device_cfg = {
+            'device_type': 'netmiko_juniper'
+        }
+        switch = juniper.Juniper(device_cfg)
+        switch.unplug_switch_from_network(10100, 100)
+        mock_exec.assert_called_with(
+            switch,
+            ['delete vlans vlan100 vxlan vni'])
+
+    @mock.patch.object(juniper.Juniper, '_get_vlan_name_by_id',
+                       return_value='vlan100', autospec=True)
+    def test_unplug_switch_from_network_evpn_without_bgp_asn(
+            self, mock_get_vlan):
+        """Test unplug_switch_from_network fails when EVPN enabled, no ASN."""
+        device_cfg = {
+            'device_type': 'netmiko_juniper',
+            'ngs_evpn_vni_config': 'true'
+        }
+        switch = juniper.Juniper(device_cfg)
+        self.assertRaises(exc.GenericSwitchNetmikoConfigError,
+                          switch.unplug_switch_from_network, 10100, 100)
